@@ -1,5 +1,5 @@
 """
-Example generation for pystruct. Stolen from scikit-learn.
+Example generation for pystruct, stolen from scikit-learn
 
 Generate the rst files for the examples by iterating over the python
 example files.
@@ -7,35 +7,68 @@ example files.
 Files that generate images should start with 'plot'
 
 """
+from __future__ import division, print_function
 from time import time
 import os
+import re
 import shutil
 import traceback
 import glob
 import sys
-from StringIO import StringIO
-import cPickle
-import re
-import urllib2
 import gzip
 import posixpath
 
-try:
-    from PIL import Image
-except:
-    import Image
 
-import matplotlib
-matplotlib.use('Agg')
+# Try Python 2 first, otherwise load from Python 3
+try:
+    from StringIO import StringIO
+    import cPickle as pickle
+    import urllib2 as urllib
+    from urllib2 import HTTPError, URLError
+except ImportError:
+    from io import StringIO
+    import pickle
+    import urllib.request
+    import urllib.error
+    import urllib.parse
+    from urllib.error import HTTPError, URLError
+
+
+try:
+    # Python 2 built-in
+    execfile
+except NameError:
+    def execfile(filename, global_vars=None, local_vars=None):
+        with open(filename) as f:
+            code = compile(f.read(), filename, 'exec')
+            exec(code, global_vars, local_vars)
+
+try:
+    basestring
+except NameError:
+    basestring = str
 
 import token
 import tokenize
 import numpy as np
 
+try:
+    # make sure that the Agg backend is set before importing any
+    # matplotlib
+    import matplotlib
+    matplotlib.use('Agg')
+except ImportError:
+    # this script can be imported by nosetest to find tests to run: we should not
+    # impose the matplotlib requirement in that case.
+    pass
+
+
+from sklearn.externals import joblib
 
 ###############################################################################
 # A tee object to redict streams to multiple outputs
 
+
 class Tee(object):
 
     def __init__(self, file1, file2):
@@ -54,11 +87,16 @@ class Tee(object):
 # Documentation link resolver objects
 
 
-def get_data(url):
+def _get_data(url):
     """Helper function to get data over http or from a local file"""
     if url.startswith('http://'):
-        resp = urllib2.urlopen(url)
-        encoding = resp.headers.dict.get('content-encoding', 'plain')
+        # Try Python 2, use Python 3 on exception
+        try:
+            resp = urllib.urlopen(url)
+            encoding = resp.headers.dict.get('content-encoding', 'plain')
+        except AttributeError:
+            resp = urllib.request.urlopen(url)
+            encoding = resp.headers.get('content-encoding', 'plain')
         data = resp.read()
         if encoding == 'plain':
             pass
@@ -74,6 +112,9 @@ def get_data(url):
 
     return data
 
+mem = joblib.Memory(cachedir='_build')
+get_data = mem.cache(_get_data)
+
 
 def parse_sphinx_searchindex(searchindex):
     """Parse a Sphinx search index
@@ -146,6 +187,10 @@ def parse_sphinx_searchindex(searchindex):
 
         return dict_out
 
+    # Make sure searchindex uses UTF-8 encoding
+    if hasattr(searchindex, 'decode'):
+        searchindex = searchindex.decode('UTF-8')
+
     # parse objects
     query = 'objects:'
     pos = searchindex.find(query)
@@ -203,7 +248,7 @@ class SphinxDocLinkResolver(object):
         if os.name.lower() == 'nt' and not doc_url.startswith('http://'):
             if not relative:
                 raise ValueError('You have to use relative=True for the local'
-                                 'package on a Windows system.')
+                                 ' package on a Windows system.')
             self._is_windows = True
         else:
             self._is_windows = False
@@ -222,7 +267,7 @@ class SphinxDocLinkResolver(object):
         if full_name in self._searchindex['objects']:
             value = self._searchindex['objects'][full_name]
             if isinstance(value, dict):
-                value = value[value.keys()[0]]
+                value = value[next(iter(value.keys()))]
             fname_idx = value[0]
         elif cobj['module_short'] in self._searchindex['objects']:
             value = self._searchindex['objects'][cobj['module_short']]
@@ -238,6 +283,9 @@ class SphinxDocLinkResolver(object):
             else:
                 link = posixpath.join(self.doc_url, fname)
 
+            if hasattr(link, 'decode'):
+                link = link.decode('utf-8', 'replace')
+
             if link in self._page_cache:
                 html = self._page_cache[link]
             else:
@@ -250,9 +298,16 @@ class SphinxDocLinkResolver(object):
                 for mod in self.extra_modules_test:
                     comb_names.append(mod + '.' + cobj['name'])
             url = False
+            if hasattr(html, 'decode'):
+                # Decode bytes under Python 3
+                html = html.decode('utf-8', 'replace')
+
             for comb_name in comb_names:
-                if html.find(comb_name) >= 0:
-                    url = link + '#' + comb_name
+                if hasattr(comb_name, 'decode'):
+                    # Decode bytes under Python 3
+                    comb_name = comb_name.decode('utf-8', 'replace')
+                if comb_name in html:
+                    url = link + u'#' + comb_name
             link = url
         else:
             link = False
@@ -332,6 +387,7 @@ plot_rst_template = """
     :lines: %(end_row)s-
 
 **Total running time of the example:** %(time_elapsed) .2f seconds
+
     """
 
 # The following strings are used when we have several pictures: we use
@@ -415,7 +471,7 @@ def generate_example_rst(app):
 
     .figure {
         float: left;
-        margin: 20px;
+        margin: 16px;
         top: 0;
         left: 0;
         -webkit-border-radius: 10px; /* Saf3-4, iOS 1-3.2, Android <1.6 */
@@ -430,10 +486,7 @@ def generate_example_rst(app):
         background-repeat: no-repeat;
         /* --> Thumbnail image size */
         width: 150px;
-        height: 200px;
-        -webkit-background-size: 150px 100px; /* Saf3-4 */
-        -moz-background-size: 150px 100px; /* FF3.6 */
-        background-size: 150px 100px; /* Opera, IE9, Saf5, Chrome, FF4 */
+        height: 130px;
     }
 
     .figure img {
@@ -441,7 +494,6 @@ def generate_example_rst(app):
         }
 
     .figure .caption {
-        width: 150px;
         text-align: center !important;
     }
     </style>
@@ -462,35 +514,40 @@ Examples
 def extract_line_count(filename, target_dir):
     # Extract the line count of a file
     example_file = os.path.join(target_dir, filename)
-    lines = file(example_file).readlines()
+    lines = open(example_file).readlines()
     start_row = 0
-    if lines[0].startswith('#!'):
+    if lines and lines[0].startswith('#!'):
         lines.pop(0)
         start_row = 1
-    tokens = tokenize.generate_tokens(lines.__iter__().next)
+    line_iterator = iter(lines)
+    tokens = tokenize.generate_tokens(lambda: next(line_iterator))
     check_docstring = True
     erow_docstring = 0
     for tok_type, _, _, (erow, _), _ in tokens:
         tok_type = token.tok_name[tok_type]
         if tok_type in ('NEWLINE', 'COMMENT', 'NL', 'INDENT', 'DEDENT'):
             continue
-        elif ((tok_type == 'STRING') and (check_docstring == True)):
+        elif (tok_type == 'STRING') and check_docstring:
             erow_docstring = erow
             check_docstring = False
     return erow_docstring+1+start_row, erow+1+start_row
 
+
 def line_count_sort(file_list, target_dir):
     # Sort the list of examples by line-count
-    new_list = filter(lambda x: x.endswith('.py'), file_list)
+    new_list = [x for x in file_list if x.endswith('.py')]
     unsorted = np.zeros(shape=(len(new_list), 2))
     unsorted = unsorted.astype(np.object)
     for count, exmpl in enumerate(new_list):
         docstr_lines, total_lines = extract_line_count(exmpl, target_dir)
         unsorted[count][1] = total_lines - docstr_lines
         unsorted[count][0] = exmpl
-    index = np.lexsort((unsorted[:,0].astype(np.str),
-                        unsorted[:,1].astype(np.float)))
-    return np.array(unsorted[index][:,0]).tolist()
+    index = np.lexsort((unsorted[:, 0].astype(np.str),
+                        unsorted[:, 1].astype(np.float)))
+    if not len(unsorted):
+        return []
+    return np.array(unsorted[index][:, 0]).tolist()
+
 
 def generate_dir_rst(dir, fhindex, example_dir, root_dir, plot_gallery):
     """ Generate the rst file for an example directory.
@@ -502,19 +559,16 @@ def generate_dir_rst(dir, fhindex, example_dir, root_dir, plot_gallery):
         target_dir = root_dir
         src_dir = example_dir
     if not os.path.exists(os.path.join(src_dir, 'README.txt')):
-        print 80 * '_'
-        print ('Example directory %s does not have a README.txt file'
-                        % src_dir)
-        print 'Skipping this directory'
-        print 80 * '_'
-        return
+        raise ValueError('Example directory %s does not have a README.txt' %
+                         src_dir)
+
     fhindex.write("""
 
 
 %s
 
 
-""" % file(os.path.join(src_dir, 'README.txt')).read())
+""" % open(os.path.join(src_dir, 'README.txt')).read())
     if not os.path.exists(target_dir):
         os.makedirs(target_dir)
     sorted_listdir = line_count_sort(os.listdir(src_dir),
@@ -569,6 +623,11 @@ def make_thumbnail(in_fname, out_fname, width, height):
     """Make a thumbnail with the same aspect ratio centered in an
        image with a given width and height
     """
+    # local import to avoid testing dependency on PIL:
+    try:
+        from PIL import Image
+    except ImportError:
+        import Image
     img = Image.open(in_fname)
     width_in, height_in = img.size
     scale_w = width / float(width_in)
@@ -587,7 +646,7 @@ def make_thumbnail(in_fname, out_fname, width, height):
 
     # insert centered
     thumb = Image.new('RGB', (width, height), (255, 255, 255))
-    pos_insert = ((width - width_sc) / 2, (height - height_sc) / 2)
+    pos_insert = ((width - width_sc) // 2, (height - height_sc) // 2)
     thumb.paste(img, pos_insert)
 
     thumb.save(out_fname)
@@ -639,7 +698,7 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
     stdout_path = os.path.join(image_dir,
                                'stdout_%s.txt' % base_image_name)
     time_path = os.path.join(image_dir,
-                               'time_%s.txt' % base_image_name)
+                             'time_%s.txt' % base_image_name)
     thumb_file = os.path.join(thumb_dir, fname[:-3] + '.png')
     time_elapsed = 0
     if plot_gallery and fname.startswith('plot'):
@@ -654,11 +713,10 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
         if os.path.exists(time_path):
             time_elapsed = float(open(time_path).read())
 
-        if (not os.path.exists(first_image_file) or
-                os.stat(first_image_file).st_mtime <=
-                                    os.stat(src_file).st_mtime):
+        if not os.path.exists(first_image_file) or \
+           os.stat(first_image_file).st_mtime <= os.stat(src_file).st_mtime:
             # We need to execute the code
-            print 'plotting %s' % fname
+            print('plotting %s' % fname)
             t0 = time()
             import matplotlib.pyplot as plt
             plt.close('all')
@@ -739,8 +797,8 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
                     # save the dictionary, so we can later add hyperlinks
                     codeobj_fname = example_file[:-3] + '_codeobj.pickle'
                     with open(codeobj_fname, 'wb') as fid:
-                        cPickle.dump(example_code_obj, fid,
-                                     cPickle.HIGHEST_PROTOCOL)
+                        pickle.dump(example_code_obj, fid,
+                                     pickle.HIGHEST_PROTOCOL)
                     fid.close()
 
                 if '__doc__' in my_globals:
@@ -763,27 +821,28 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
                 #    incrementally: 1, 2, 3 and not 1, 2, 5)
                 # * iterate over [fig_mngr.num for fig_mngr in
                 #   matplotlib._pylab_helpers.Gcf.get_all_fig_managers()]
-                for fig_num in (fig_mngr.num for fig_mngr in
-                        matplotlib._pylab_helpers.Gcf.get_all_fig_managers()):
+                fig_managers = matplotlib._pylab_helpers.Gcf.get_all_fig_managers()
+                for fig_mngr in fig_managers:
                     # Set the fig_num figure as the current figure as we can't
                     # save a figure that's not the current figure.
-                    plt.figure(fig_num)
-                    plt.savefig(image_path % fig_num)
-                    figure_list.append(image_fname % fig_num)
+                    plt.figure(fig_mngr.num)
+                    plt.savefig(image_path % fig_mngr.num)
+                    figure_list.append(image_fname % fig_mngr.num)
             except:
-                print 80 * '_'
-                print '%s is not compiling:' % fname
+                print(80 * '_')
+                print('%s is not compiling:' % fname)
                 traceback.print_exc()
-                print 80 * '_'
+                print(80 * '_')
             finally:
                 os.chdir(cwd)
                 sys.stdout = orig_stdout
 
-            print " - time elapsed : %.2g sec" % time_elapsed
+            print(" - time elapsed : %.2g sec" % time_elapsed)
         else:
             figure_list = [f[len(image_dir):]
-                            for f in glob.glob(image_path % '[1-9]')]
-                            #for f in glob.glob(image_path % '*')]
+                           for f in glob.glob(image_path.replace("%03d",
+                                                '[0-9][0-9][0-9]'))]
+        figure_list.sort()
 
         # generate thumb file
         this_template = plot_rst_template
@@ -813,84 +872,98 @@ def generate_file_rst(fname, target_dir, src_dir, plot_gallery):
 
 def embed_code_links(app, exception):
     """Embed hyperlinks to documentation into example code"""
-    try:
-        if exception is not None:
-            return
-        print 'Embedding documentation hyperlinks in examples..'
+    if exception is not None:
+        return
+    print('Embedding documentation hyperlinks in examples..')
 
-        # Add resolvers for the packages for which we want to show links
-        doc_resolvers = {}
-        doc_resolvers['sklearn'] = SphinxDocLinkResolver(app.builder.outdir,
-                                                         relative=True)
+    # Add resolvers for the packages for which we want to show links
+    doc_resolvers = {}
+    doc_resolvers['pystruct'] = SphinxDocLinkResolver(app.builder.outdir,
+                                                     relative=True)
 
-        doc_resolvers['matplotlib'] = SphinxDocLinkResolver(
-            'http://matplotlib.org')
+    resolver_urls = {
+        'sklearn': 'http://scikit-learn.org/stable',
+        'matplotlib': 'http://matplotlib.org',
+        'numpy': 'http://docs.scipy.org/doc/numpy-1.6.0',
+        'scipy': 'http://docs.scipy.org/doc/scipy-0.11.0/reference',
+    }
+    for this_module, url in resolver_urls.items():
+        try:
+            doc_resolvers[this_module] = SphinxDocLinkResolver(url)
+        except HTTPError as e:
+            print("The following HTTP Error has occurred:\n")
+            print(e.code)
+        except URLError as e:
+            print("\n...\n"
+                  "Warning: Embedding the documentation hyperlinks requires "
+                  "internet access.\nPlease check your network connection.\n"
+                  "Unable to continue embedding `{0}` links due to a URL "
+                  "Error:\n".format(this_module))
+            print(e.args)
 
-        doc_resolvers['numpy'] = SphinxDocLinkResolver(
-            'http://docs.scipy.org/doc/numpy-1.6.0')
+    example_dir = os.path.join(app.builder.srcdir, 'auto_examples')
+    html_example_dir = os.path.abspath(os.path.join(app.builder.outdir,
+                                                    'auto_examples'))
 
-        doc_resolvers['scipy'] = SphinxDocLinkResolver(
-            'http://docs.scipy.org/doc/scipy-0.11.0/reference')
+    # patterns for replacement
+    link_pattern = '<a href="%s">%s</a>'
+    orig_pattern = '<span class="n">%s</span>'
+    period = '<span class="o">.</span>'
 
-        example_dir = os.path.join(app.builder.srcdir, 'auto_examples')
-        html_example_dir = os.path.abspath(os.path.join(app.builder.outdir,
-                                                        'auto_examples'))
+    for dirpath, _, filenames in os.walk(html_example_dir):
+        for fname in filenames:
+            print('\tprocessing: %s' % fname)
+            full_fname = os.path.join(html_example_dir, dirpath, fname)
+            subpath = dirpath[len(html_example_dir) + 1:]
+            pickle_fname = os.path.join(example_dir, subpath,
+                                        fname[:-5] + '_codeobj.pickle')
 
-        # patterns for replacement
-        link_pattern = '<a href="%s">%s</a>'
-        orig_pattern = '<span class="n">%s</span>'
-        period = '<span class="o">.</span>'
+            if os.path.exists(pickle_fname):
+                # we have a pickle file with the objects to embed links for
+                with open(pickle_fname, 'rb') as fid:
+                    example_code_obj = pickle.load(fid)
+                fid.close()
+                str_repl = {}
+                # generate replacement strings with the links
+                for name, cobj in example_code_obj.items():
+                    this_module = cobj['module'].split('.')[0]
 
-        for dirpath, _, filenames in os.walk(html_example_dir):
-            for fname in filenames:
-                print '\tprocessing: %s' % fname
-                full_fname = os.path.join(html_example_dir, dirpath, fname)
-                subpath = dirpath[len(html_example_dir) + 1:]
-                pickle_fname = os.path.join(example_dir, subpath,
-                                            fname[:-5] + '_codeobj.pickle')
-
-                if os.path.exists(pickle_fname):
-                    # we have a pickle file with the objects to embed links for
-                    with open(pickle_fname, 'rb') as fid:
-                        example_code_obj = cPickle.load(fid)
-                    fid.close()
-                    str_repl = {}
-                    # generate replacement strings with the links
-                    for name, cobj in example_code_obj.iteritems():
-                        this_module = cobj['module'].split('.')[0]
-
-                        if this_module not in doc_resolvers:
-                            continue
+                    if this_module not in doc_resolvers:
+                        continue
 
+                    try:
                         link = doc_resolvers[this_module].resolve(cobj,
                                                                   full_fname)
-                        if link is not None:
-                            parts = name.split('.')
-                            name_html = orig_pattern % parts[0]
-                            for part in parts[1:]:
-                                name_html += period + orig_pattern % part
-                            str_repl[name_html] = link_pattern % (link, name_html)
-                    # do the replacement in the html file
-                    if len(str_repl) > 0:
-                        with open(full_fname, 'rt') as fid:
-                            lines_in = fid.readlines()
-                        fid.close()
-                        with open(full_fname, 'wt') as fid:
-                            for line in lines_in:
-                                for name, link in str_repl.iteritems():
-                                    line = line.replace(name, link)
-                                fid.write(line)
-                        fid.close()
-    except urllib2.HTTPError, e:
-        print ("The following HTTP Error has occurred:\n")
-        print e.code
-    except urllib2.URLError, e:
-        print ("\n...\n"
-               "Warning: Embedding the documentation hyperlinks requires "
-               "internet access.\nPlease check your network connection.\n"
-               "Unable to continue embedding due to a URL Error: \n")
-        print e.args
-    print '[done]'
+                    except (HTTPError, URLError) as e:
+                        print("The following error has occurred:\n")
+                        print(repr(e))
+                        continue
+
+                    if link is not None:
+                        parts = name.split('.')
+                        name_html = period.join(orig_pattern % part
+                                                for part in parts)
+                        str_repl[name_html] = link_pattern % (link, name_html)
+                # do the replacement in the html file
+
+                # ensure greediness
+                names = sorted(str_repl, key=len, reverse=True)
+                expr = re.compile(r'(?<!\.)\b' +  # don't follow . or word
+                                  '|'.join(re.escape(name)
+                                           for name in names))
+
+                def substitute_link(match):
+                    return str_repl[match.group()]
+
+                if len(str_repl) > 0:
+                    with open(full_fname, 'rb') as fid:
+                        lines_in = fid.readlines()
+                    with open(full_fname, 'wb') as fid:
+                        for line in lines_in:
+                            line = line.decode('utf-8')
+                            line = expr.sub(substitute_link, line)
+                            fid.write(line.encode('utf-8'))
+    print('[done]')
 
 
 def setup(app):
@@ -902,7 +975,7 @@ def setup(app):
 
     # Sphinx hack: sphinx copies generated images to the build directory
     #  each time the docs are made.  If the desired image name already
-    #  exists, it appends a digit to prevent overwrites.  The model is,
+    #  exists, it appends a digit to prevent overwrites.  The problem is,
     #  the directory is never cleared.  This means that each time you build
     #  the docs, the number of images in the directory grows.
     #
@@ -920,3 +993,8 @@ def setup(app):
         for filename in filelist:
             if filename.endswith('png'):
                 os.remove(os.path.join(build_image_dir, filename))
+
+
+def setup_module():
+    # HACK: Stop nosetests running setup() above
+    pass
