@@ -1,11 +1,13 @@
 import numpy as np
 
 from .linear_programming import lp_general_graph
+from .maxprod import inference_max_product
+from .common import _validate_params, compute_energy
 
 
 def get_installed(method_filter=None):
     if method_filter is None:
-        method_filter = ['ad3', 'qpbo', 'dai', 'ogm', 'lp']
+        method_filter = ["max-product", 'ad3', 'qpbo', 'dai', 'ogm', 'lp']
 
     installed = []
     unary = np.zeros((1, 1))
@@ -20,40 +22,6 @@ def get_installed(method_filter=None):
     return installed
 
 
-def compute_energy(unary_potentials, pairwise_potentials, edges, labels):
-    """Compute energy of labels for given energy function.
-
-    Convenience function with same interface as inference functions to easily
-    compare solutions.
-
-    Parameters
-    ----------
-    unary_potentials : nd-array
-        Unary potentials of energy function.
-
-    pairwise_potentials : nd-array
-        Pairwise potentials of energy function.
-
-    edges : nd-array
-        Edges of energy function.
-
-    labels : nd-array
-        Variable assignment to evaluate.
-
-    Returns
-    -------
-    energy : float
-        Energy of assignment.
-    """
-
-    n_states, pairwise_potentials = \
-        _validate_params(unary_potentials, pairwise_potentials, edges)
-    energy = np.sum(unary_potentials[np.arange(len(labels)), labels])
-    for edge, pw in zip(edges, pairwise_potentials):
-        energy += pw[labels[edge[0]], labels[edge[1]]]
-    return energy
-
-
 def inference_dispatch(unary_potentials, pairwise_potentials, edges,
                        inference_method, return_energy=False, **kwargs):
     """Wrapper function to dispatch between inference method by string.
@@ -73,9 +41,10 @@ def inference_dispatch(unary_potentials, pairwise_potentials, edges,
         Possible choices currently are:
             * 'qpbo' for QPBO alpha-expansion (fast but approximate).
             * 'dai' for libDAI wrappers (default to junction tree).
-            * 'lp' for build-in lp relaxation via GLPK (slow).
+            * 'lp' for build-in lp relaxation via cvxopt (slow).
             * 'ad3' for AD^3 subgradient based dual solution of LP.
             * 'ogm' for OpenGM wrappers.
+            * 'max-product' for max-product message passing.
             * 'unary' for using unary potentials only.
 
         It is also possible to pass a tuple (string, dict) where the dict
@@ -122,27 +91,12 @@ def inference_dispatch(unary_potentials, pairwise_potentials, edges,
     elif inference_method == "unary":
         return inference_unaries(unary_potentials, pairwise_potentials, edges,
                                  **kwargs)
+    elif inference_method == "max-product":
+        return inference_max_product(unary_potentials, pairwise_potentials,
+                                     edges, **kwargs)
     else:
-        raise ValueError("inference_method must be 'lp', 'ad3', 'qpbo', 'ogm'"
-                         " or 'dai', got %s" % inference_method)
-
-
-def _validate_params(unary_potentials, pairwise_params, edges):
-    n_states = unary_potentials.shape[-1]
-    if pairwise_params.shape == (n_states, n_states):
-        # only one matrix given
-        pairwise_potentials = np.repeat(pairwise_params[np.newaxis, :, :],
-                                        edges.shape[0], axis=0)
-    else:
-        if pairwise_params.shape != (edges.shape[0], n_states, n_states):
-            raise ValueError("Expected pairwise_params either to "
-                             "be of shape n_states x n_states "
-                             "or n_edges x n_states x n_states, but"
-                             " got shape %s. n_states=%d, n_edge=%d."
-                             % (repr(pairwise_params.shape), n_states,
-                                edges.shape[0]))
-        pairwise_potentials = pairwise_params
-    return n_states, pairwise_potentials
+        raise ValueError("inference_method must be 'max-product', 'lp', 'ad3',"
+                         " 'qpbo', 'ogm' or 'dai', got %s" % inference_method)
 
 
 def inference_ogm(unary_potentials, pairwise_potentials, edges,
@@ -195,15 +149,15 @@ def inference_ogm(unary_potentials, pairwise_potentials, edges,
         _validate_params(unary_potentials, pairwise_potentials, edges)
     n_nodes = len(unary_potentials)
 
-    gm = opengm.gm(np.ones(n_nodes, dtype=opengm.label_type)*n_states)
+    gm = opengm.gm(np.ones(n_nodes, dtype=opengm.label_type) * n_states)
 
-    nFactors = int(n_nodes+edges.shape[0])
+    nFactors = int(n_nodes + edges.shape[0])
     gm.reserveFactors(nFactors)
     gm.reserveFunctions(nFactors, 'explicit')
 
     # all unaries as one numpy array
     # (opengm's value_type == float64 but all types are accepted)
-    unaries = np.require(unary_potentials, dtype=opengm.value_type)*-1.0
+    unaries = np.require(unary_potentials, dtype=opengm.value_type) * -1.0
     # add all unart functions at once
     fidUnaries = gm.addFunctions(unaries)
     visUnaries = np.arange(n_nodes, dtype=opengm.label_type)
@@ -337,7 +291,7 @@ def inference_dai(unary_potentials, pairwise_potentials, edges,
 
 def inference_lp(unary_potentials, pairwise_potentials, edges, relaxed=False,
                  return_energy=False, **kwargs):
-    """Inference with build-in LP solver using GLPK backend.
+    """Inference with build-in LP solver using cvxopt backend.
 
     Parameters
     ----------
@@ -372,9 +326,6 @@ def inference_lp(unary_potentials, pairwise_potentials, edges, relaxed=False,
     unaries = unary_potentials.reshape(-1, n_states)
     res = lp_general_graph(-unaries, edges, -pairwise_potentials)
     unary_marginals, pairwise_marginals, energy = res
-    #n_fractional = np.sum(unary_marginals.max(axis=-1) < .99)
-    #if n_fractional:
-        #print("fractional solutions found: %d" % n_fractional)
     if relaxed:
         unary_marginals = unary_marginals.reshape(unary_potentials.shape)
         y = (unary_marginals, pairwise_marginals)
@@ -428,7 +379,7 @@ def inference_ad3(unary_potentials, pairwise_potentials, edges, relaxed=False,
         _validate_params(unary_potentials, pairwise_potentials, edges)
 
     unaries = unary_potentials.reshape(-1, n_states)
-    res = ad3.general_graph(unaries, edges, pairwise_potentials, verbose=1,
+    res = ad3.general_graph(unaries, edges, pairwise_potentials, verbose=verbose,
                             n_iterations=4000, exact=branch_and_bound)
     unary_marginals, pairwise_marginals, energy, solver_status = res
     if verbose:
