@@ -85,9 +85,13 @@ class NSlackSSVM(BaseSSVM):
         Switch to the given inference method if the previous method does not
         find any more constraints.
 
+    logger : logger object, default=None
+        Pystruct logger for storing the model or extracting additional
+        information.
+
     Attributes
     ----------
-    w : nd-array, shape=(model.size_psi,)
+    w : nd-array, shape=(model.size_joint_feature,)
         The learned weights of the SVM.
 
     old_solution : dict
@@ -104,6 +108,15 @@ class NSlackSSVM(BaseSSVM):
 
     ``timestamps_`` : list of int
        Total training time stored before each iteration.
+
+    References
+    ----------
+    * Tsochantaridis, Ioannis and Joachims, Thorsten and Hofmann, Thomas and
+        Altun, Yasemin and Singer, Yoram: Large margin methods for structured
+        and interdependent output variables, JMLR 2006
+
+    * Joachims, Thorsten and Finley, Thomas and Yu, Chun-Nam John:
+        Cutting-plane training of structural SVMs, JMLR 2009
     """
 
     def __init__(self, model, max_iter=100, C=1.0, check_constraints=True,
@@ -127,12 +140,12 @@ class NSlackSSVM(BaseSSVM):
 
     def _solve_n_slack_qp(self, constraints, n_samples):
         C = self.C
-        psis = [c[1] for sample in constraints for c in sample]
+        joint_features = [c[1] for sample in constraints for c in sample]
         losses = [c[2] for sample in constraints for c in sample]
 
-        psi_matrix = np.vstack(psis).astype(np.float)
-        n_constraints = len(psis)
-        P = cvxopt.matrix(np.dot(psi_matrix, psi_matrix.T))
+        joint_feature_matrix = np.vstack(joint_features).astype(np.float)
+        n_constraints = len(joint_features)
+        P = cvxopt.matrix(np.dot(joint_feature_matrix, joint_feature_matrix.T))
         # q contains loss from margin-rescaling
         q = cvxopt.matrix(-np.array(losses, dtype=np.float))
         # constraints are a bit tricky. first, all alpha must be >zero
@@ -148,24 +161,27 @@ class NSlackSSVM(BaseSSVM):
         if self.negativity_constraint is None:
             #empty constraints
             zero_constr = np.zeros(0)
-            psis_constr = np.zeros((0, n_constraints))
+            joint_features_constr = np.zeros((0, n_constraints))
         else:
-            psis_constr = psi_matrix.T[self.negativity_constraint]
+            joint_features_constr = joint_feature_matrix.T[self.negativity_constraint]
             zero_constr = np.zeros(len(self.negativity_constraint))
 
         # put together
         G = cvxopt.sparse(cvxopt.matrix(np.vstack((-idy, blocks,
-                                                   psis_constr))))
+                                                   joint_features_constr))))
         tmp2 = np.ones(n_samples) * C
         h = cvxopt.matrix(np.hstack((tmp1, tmp2, zero_constr)))
 
         # solve QP model
         cvxopt.solvers.options['feastol'] = 1e-5
-        solution = cvxopt.solvers.qp(P, q, G, h)
+        try:
+            solution = cvxopt.solvers.qp(P, q, G, h)
+        except ValueError:
+            solution = {'status': 'error'}
         if solution['status'] != "optimal":
             print("regularizing QP!")
-            P = cvxopt.matrix(np.dot(psi_matrix, psi_matrix.T)
-                              + 1e-8 * np.eye(psi_matrix.shape[0]))
+            P = cvxopt.matrix(np.dot(joint_feature_matrix, joint_feature_matrix.T)
+                              + 1e-8 * np.eye(joint_feature_matrix.shape[0]))
             solution = cvxopt.solvers.qp(P, q, G, h)
             if solution['status'] != "optimal":
                 raise ValueError("QP solver failed. Try regularizing your QP.")
@@ -184,7 +200,7 @@ class NSlackSSVM(BaseSSVM):
             # calculate per example box constraint:
             print("Box constraints at C: %d" % np.sum(1 - box / C < 1e-3))
             print("dual objective: %f" % -solution['primal objective'])
-        self.w = np.dot(a, psi_matrix)
+        self.w = np.dot(a, joint_feature_matrix)
         return -solution['primal objective']
 
     def _check_bad_constraint(self, y_hat, slack, old_constraints):
@@ -237,8 +253,8 @@ class NSlackSSVM(BaseSSVM):
         contraints : iterable
             Known constraints for warm-starts. List of same length as X.
             Each entry is itself a list of constraints for a given instance x .
-            Each constraint is of the form [y_hat, delta_psi, loss], where
-            y_hat is a labeling, ``delta_psi = psi(x, y) - psi(x, y_hat)``
+            Each constraint is of the form [y_hat, delta_joint_feature, loss], where
+            y_hat is a labeling, ``delta_joint_feature = joint_feature(x, y) - joint_feature(x, y_hat)``
             and loss is the loss for predicting y_hat instead of the true label
             y.
 
@@ -246,11 +262,12 @@ class NSlackSSVM(BaseSSVM):
             Whether to initialize the model for the data.
             Leave this true except if you really know what you are doing.
         """
-        print("Training n-slack dual structural SVM")
+        if self.verbose:
+            print("Training n-slack dual structural SVM")
         cvxopt.solvers.options['show_progress'] = self.verbose > 3
         if initialize:
             self.model.initialize(X, Y)
-        self.w = np.zeros(self.model.size_psi)
+        self.w = np.zeros(self.model.size_joint_feature)
         n_samples = len(X)
         stopping_criterion = False
         if constraints is None:
@@ -299,7 +316,7 @@ class NSlackSSVM(BaseSSVM):
                     for i, x, y, constraint in zip(indices_b, X_b, Y_b,
                                                    candidate_constraints):
                         # loop over samples in batch
-                        y_hat, delta_psi, slack, loss = constraint
+                        y_hat, delta_joint_feature, slack, loss = constraint
                         slack_sum += slack
 
                         if self.verbose > 3:
@@ -307,14 +324,14 @@ class NSlackSSVM(BaseSSVM):
 
                         if not loss > 0:
                             # can have y != y_hat but loss = 0 in latent svm.
-                            # we need this here as dpsi is then != 0
+                            # we need this here as djoint_feature is then != 0
                             continue
 
                         if self._check_bad_constraint(y_hat, slack,
                                                       constraints[i]):
                             continue
 
-                        constraints[i].append([y_hat, delta_psi, loss])
+                        constraints[i].append([y_hat, delta_joint_feature, loss])
                         new_constraints_batch += 1
 
                     # after processing the slice, solve the qp
@@ -337,20 +354,22 @@ class NSlackSSVM(BaseSSVM):
                           (new_constraints, objective, primal_objective))
 
                 if new_constraints == 0:
-                    print("no additional constraints")
+                    if self.verbose:
+                        print("no additional constraints")
                     stopping_criterion = True
 
-                if (not stopping_criterion and iteration > 1 and
-                    self.objective_curve_[-1]
+                if (iteration > 1 and self.objective_curve_[-1]
                         - self.objective_curve_[-2] < self.tol):
-                    print("objective converged.")
+                    if self.verbose:
+                        print("objective converged.")
                     stopping_criterion = True
 
                 if stopping_criterion:
                     if (self.switch_to is not None and
                             self.model.inference_method != self.switch_to):
-                        print("Switching to %s inference" %
-                              str(self.switch_to))
+                        if self.verbose:
+                            print("Switching to %s inference" %
+                                  str(self.switch_to))
                         self.model.inference_method_ = \
                             self.model.inference_method
                         self.model.inference_method = self.switch_to
@@ -366,12 +385,18 @@ class NSlackSSVM(BaseSSVM):
                     self.logger(self, iteration)
         except KeyboardInterrupt:
             pass
-        if self.logger is not None:
-            self.logger(self, 'final')
 
         self.constraints_ = constraints
         if self.verbose and self.n_jobs == 1:
             print("calls to inference: %d" % self.model.inference_calls)
+
+        if verbose:
+            print("Computing final objective.")
+        self.timestamps_.append(time() - self.timestamps_[0])
+        self.primal_objective_curve_.append(self._objective(X, Y))
+        self.objective_curve_.append(objective)
+        if self.logger is not None:
+            self.logger(self, 'final')
         return self
 
     def prune_constraints(self, constraints, a):
