@@ -35,11 +35,11 @@ class OneSlackSSVM(BaseSSVM):
         Object containing the model structure. Has to implement
         `loss`, `inference` and `loss_augmented_inference`.
 
-    max_iter : int
+    max_iter : int, default=10000
         Maximum number of passes over dataset to find constraints.
 
-    C : float (default=1)
-        Regularization parameter
+    C : float, default=1
+        Regularization parameter.
 
     check_constraints : bool
         Whether to check if the new "most violated constraint" is
@@ -47,14 +47,14 @@ class OneSlackSSVM(BaseSSVM):
         and debugging, but costly.
 
     verbose : int
-        Verbosity
+        Verbosity.
 
-    negativity_constraint: list of ints
+    negativity_constraint : list of ints
         Indices of parmeters that are constraint to be negative.
         This is useful for learning submodular CRFs (inference is formulated
         as maximization in SSVMs, flipping some signs).
 
-    break_on_bad: bool (default=False)
+    break_on_bad : bool default=False
         Whether to break (start debug mode) when inference was approximate.
 
     n_jobs : int, default=1
@@ -97,9 +97,13 @@ class OneSlackSSVM(BaseSSVM):
         Switch to the given inference method if the previous method does not
         find any more constraints.
 
+    logger : logger object, default=None
+        Pystruct logger for storing the model or extracting additional
+        information.
+
     Attributes
     ----------
-    w : nd-array, shape=(model.size_psi,)
+    w : nd-array, shape=(model.size_joint_feature,)
         The learned weights of the SVM.
 
     old_solution : dict
@@ -116,6 +120,12 @@ class OneSlackSSVM(BaseSSVM):
 
     ``timestamps_`` : list of int
        Total training time stored before each iteration.
+
+    References
+    ----------
+    * Joachims, Thorsten and Finley, Thomas and Yu, Chun-Nam John:
+        Cutting-plane training of structural SVMs, JMLR 2009
+
     """
 
     def __init__(self, model, max_iter=10000, C=1.0, check_constraints=False,
@@ -141,12 +151,12 @@ class OneSlackSSVM(BaseSSVM):
 
     def _solve_1_slack_qp(self, constraints, n_samples):
         C = np.float(self.C) * n_samples  # this is how libsvm/svmstruct do it
-        psis = [c[0] for c in constraints]
+        joint_features = [c[0] for c in constraints]
         losses = [c[1] for c in constraints]
 
-        psi_matrix = np.vstack(psis)
-        n_constraints = len(psis)
-        P = cvxopt.matrix(np.dot(psi_matrix, psi_matrix.T))
+        joint_feature_matrix = np.vstack(joint_features)
+        n_constraints = len(joint_features)
+        P = cvxopt.matrix(np.dot(joint_feature_matrix, joint_feature_matrix.T))
         # q contains loss from margin-rescaling
         q = cvxopt.matrix(-np.array(losses, dtype=np.float))
         # constraints: all alpha must be >zero
@@ -156,13 +166,13 @@ class OneSlackSSVM(BaseSSVM):
         if self.negativity_constraint is None:
             #empty constraints
             zero_constr = np.zeros(0)
-            psis_constr = np.zeros((0, n_constraints))
+            joint_features_constr = np.zeros((0, n_constraints))
         else:
-            psis_constr = psi_matrix.T[self.negativity_constraint]
+            joint_features_constr = joint_feature_matrix.T[self.negativity_constraint]
             zero_constr = np.zeros(len(self.negativity_constraint))
 
         # put together
-        G = cvxopt.sparse(cvxopt.matrix(np.vstack((-idy, psis_constr))))
+        G = cvxopt.sparse(cvxopt.matrix(np.vstack((-idy, joint_features_constr))))
         h = cvxopt.matrix(np.hstack((tmp1, zero_constr)))
 
         # equality constraint: sum of all alpha must be = C
@@ -171,23 +181,14 @@ class OneSlackSSVM(BaseSSVM):
 
         # solve QP model
         cvxopt.solvers.options['feastol'] = 1e-5
-        #if hasattr(self, 'old_solution'):
-            #s = self.old_solution['s']
-            ## put s slightly inside the cone..
-            #s = cvxopt.matrix(np.vstack([s, [[1e-10]]]))
-            #z = self.old_solution['z']
-            #z = cvxopt.matrix(np.vstack([z, [[1e-10]]]))
-            #initvals = {'x': self.old_solution['x'], 'y':
-                        #self.old_solution['y'], 'z': z,
-                        #'s': s}
-        #else:
-            #initvals = {}
-        #solution = cvxopt.solvers.qp(P, q, G, h, A, b, initvals=initvals)
-        solution = cvxopt.solvers.qp(P, q, G, h, A, b)
+        try:
+            solution = cvxopt.solvers.qp(P, q, G, h, A, b)
+        except ValueError:
+            solution = {'status': 'error'}
         if solution['status'] != "optimal":
             print("regularizing QP!")
-            P = cvxopt.matrix(np.dot(psi_matrix, psi_matrix.T)
-                              + 1e-8 * np.eye(psi_matrix.shape[0]))
+            P = cvxopt.matrix(np.dot(joint_feature_matrix, joint_feature_matrix.T)
+                              + 1e-8 * np.eye(joint_feature_matrix.shape[0]))
             solution = cvxopt.solvers.qp(P, q, G, h, A, b)
             if solution['status'] != "optimal":
                 raise ValueError("QP solver failed. Try regularizing your QP.")
@@ -202,7 +203,7 @@ class OneSlackSSVM(BaseSSVM):
         if self.verbose > 1:
             print("%d support vectors out of %d points" % (np.sum(sv),
                                                            n_constraints))
-        self.w = np.dot(a, psi_matrix)
+        self.w = np.dot(a, joint_feature_matrix)
         # we needed to flip the sign to make the dual into a minimization
         # model
         return -solution['primal objective']
@@ -230,7 +231,7 @@ class OneSlackSSVM(BaseSSVM):
                 del constraints[idx]
                 del self.alphas[idx]
 
-    def _check_bad_constraint(self, violation, dpsi_mean, loss,
+    def _check_bad_constraint(self, violation, djoint_feature_mean, loss,
                               old_constraints, break_on_bad, tol=None):
         violation_difference = violation - self.last_slack_
         if self.verbose > 1:
@@ -242,10 +243,11 @@ class OneSlackSSVM(BaseSSVM):
         if tol is None:
             tol = self.tol
         if violation_difference < tol:
-            print("new constraint too weak.")
+            if self.verbose:
+                print("new constraint too weak.")
             return True
-        equals = [True for dpsi_, loss_ in old_constraints
-                  if (np.all(dpsi_ == dpsi_mean) and loss == loss_)]
+        equals = [True for djoint_feature_, loss_ in old_constraints
+                  if (np.all(djoint_feature_ == djoint_feature_mean) and loss == loss_)]
 
         if np.any(equals):
             return True
@@ -260,7 +262,8 @@ class OneSlackSSVM(BaseSSVM):
                 # significantly larger, don't add constraint.
                 # if smaller, complain about approximate inference.
                 if violation - violation_tmp < -1e-5:
-                    print("bad inference: %f" % (violation_tmp - violation))
+                    if self.verbose:
+                        print("bad inference: %f" % (violation_tmp - violation))
                     if break_on_bad:
                         raise ValueError("Bad inference: new violation is"
                                          " weaker than previous constraint.")
@@ -277,7 +280,7 @@ class OneSlackSSVM(BaseSSVM):
 
         def constraint_equal(y_1, y_2):
             if isinstance(y_1, tuple):
-                return np.all(y_1[0] == y_2[1]) and np.all(y_1[1] == y_2[1])
+                return np.all(y_1[0] == y_2[0]) and np.all(y_1[1] == y_2[1])
             return np.all(y_1 == y_2)
 
         for sample, x, y, y_hat in zip(self.inference_cache_, X, Y, Y_hat):
@@ -291,47 +294,48 @@ class OneSlackSSVM(BaseSSVM):
             # this makes it a little less efficient in the caching case.
             # the idea is that if we cache, inference is way more expensive
             # and this doesn't matter much.
-            sample.append((self.model.psi(x, y_hat),
+            sample.append((self.model.joint_feature(x, y_hat),
                            self.model.loss(y, y_hat), y_hat))
 
-    def _constraint_from_cache(self, X, Y, psi_gt, constraints):
+    def _constraint_from_cache(self, X, Y, joint_feature_gt, constraints):
         if (not getattr(self, 'inference_cache_', False) or
                 self.inference_cache_ is False):
             if self.verbose > 10:
                 print("Empty cache.")
             raise NoConstraint
-        if (self.cache_tol == 'auto' and
-                (self.primal_objective_curve_[-1] - self.objective_curve_[-1])
-                < self.cache_tol_):
+        gap = self.primal_objective_curve_[-1] - self.objective_curve_[-1]
+        if (self.cache_tol == 'auto' and gap < self.cache_tol_):
             # do inference if gap has become to small
             if self.verbose > 1:
-                print("Last gap too small, not loading constraint from cache.")
+                print("Last gap too small (%f < %f), not loading constraint"
+                      " from cache."
+                      % (gap, self.cache_tol_))
             raise NoConstraint
 
         Y_hat = []
-        psi_acc = np.zeros(self.model.size_psi)
+        joint_feature_acc = np.zeros(self.model.size_joint_feature)
         loss_mean = 0
         for cached in self.inference_cache_:
-            # cached has entries of form (psi, loss, y_hat)
-            violations = [np.dot(psi, self.w) + loss
-                          for psi, loss, _ in cached]
-            psi, loss, y_hat = cached[np.argmax(violations)]
+            # cached has entries of form (joint_feature, loss, y_hat)
+            violations = [np.dot(joint_feature, self.w) + loss
+                          for joint_feature, loss, _ in cached]
+            joint_feature, loss, y_hat = cached[np.argmax(violations)]
             Y_hat.append(y_hat)
-            psi_acc += psi
+            joint_feature_acc += joint_feature
             loss_mean += loss
 
-        dpsi = (psi_gt - psi_acc) / len(X)
+        djoint_feature = (joint_feature_gt - joint_feature_acc) / len(X)
         loss_mean = loss_mean / len(X)
 
-        violation = loss_mean - np.dot(self.w, dpsi)
-        if self._check_bad_constraint(violation, dpsi, loss_mean, constraints,
+        violation = loss_mean - np.dot(self.w, djoint_feature)
+        if self._check_bad_constraint(violation, djoint_feature, loss_mean, constraints,
                                       break_on_bad=False):
             if self.verbose > 1:
                 print("No constraint from cache.")
             raise NoConstraint
-        return Y_hat, dpsi, loss_mean
+        return Y_hat, djoint_feature, loss_mean
 
-    def _find_new_constraint(self, X, Y, psi_gt, constraints, check=True):
+    def _find_new_constraint(self, X, Y, joint_feature_gt, constraints, check=True):
         if self.n_jobs != 1:
             # do inference in parallel
             verbose = max(0, self.verbose - 3)
@@ -342,21 +346,23 @@ class OneSlackSSVM(BaseSSVM):
         else:
             Y_hat = self.model.batch_loss_augmented_inference(
                 X, Y, self.w, relaxed=True)
-        # compute the mean over psis and losses
+        # compute the mean over joint_features and losses
 
         if getattr(self.model, 'rescale_C', False):
-            dpsi = (psi_gt - self.model.batch_psi(X, Y_hat, Y)) / len(X)
+            djoint_feature = (joint_feature_gt
+                              - self.model.batch_joint_feature(X, Y_hat, Y)) / len(X)
         else:
-            dpsi = (psi_gt - self.model.batch_psi(X, Y_hat)) / len(X)
+            djoint_feature = (joint_feature_gt
+                              - self.model.batch_joint_feature(X, Y_hat)) / len(X)
 
         loss_mean = np.mean(self.model.batch_loss(Y, Y_hat))
 
-        violation = loss_mean - np.dot(self.w, dpsi)
+        violation = loss_mean - np.dot(self.w, djoint_feature)
         if check and self._check_bad_constraint(
-                violation, dpsi, loss_mean, constraints,
+                violation, djoint_feature, loss_mean, constraints,
                 break_on_bad=self.break_on_bad):
             raise NoConstraint
-        return Y_hat, dpsi, loss_mean
+        return Y_hat, djoint_feature, loss_mean
 
     def fit(self, X, Y, constraints=None, warm_start=False, initialize=True):
         """Learn parameters using cutting plane method.
@@ -380,7 +386,8 @@ class OneSlackSSVM(BaseSSVM):
             Whether to initialize the model for the data.
             Leave this true except if you really know what you are doing.
         """
-        print("Training 1-slack dual structural SVM")
+        if self.verbose:
+            print("Training 1-slack dual structural SVM")
         cvxopt.solvers.options['show_progress'] = self.verbose > 3
         if initialize:
             self.model.initialize(X, Y)
@@ -392,22 +399,22 @@ class OneSlackSSVM(BaseSSVM):
             self.cache_tol_ = self.cache_tol
 
         if not warm_start:
-            self.w = np.zeros(self.model.size_psi)
+            self.w = np.zeros(self.model.size_joint_feature)
             constraints = []
             self.objective_curve_, self.primal_objective_curve_ = [], []
             self.cached_constraint_ = []
             self.alphas = []  # dual solutions
             # append constraint given by ground truth to make our life easier
-            constraints.append((np.zeros(self.model.size_psi), 0))
+            constraints.append((np.zeros(self.model.size_joint_feature), 0))
             self.alphas.append([self.C])
             self.inference_cache_ = None
             self.timestamps_ = [time()]
         elif warm_start == "soft":
-            self.w = np.zeros(self.model.size_psi)
+            self.w = np.zeros(self.model.size_joint_feature)
             constraints = []
             self.alphas = []  # dual solutions
             # append constraint given by ground truth to make our life easier
-            constraints.append((np.zeros(self.model.size_psi), 0))
+            constraints.append((np.zeros(self.model.size_joint_feature), 0))
             self.alphas.append([self.C])
 
         else:
@@ -415,11 +422,11 @@ class OneSlackSSVM(BaseSSVM):
 
         self.last_slack_ = -1
 
-        # get the psi of the ground truth
+        # get the joint_feature of the ground truth
         if getattr(self.model, 'rescale_C', False):
-            psi_gt = self.model.batch_psi(X, Y, Y)
+            joint_feature_gt = self.model.batch_joint_feature(X, Y, Y)
         else:
-            psi_gt = self.model.batch_psi(X, Y)
+            joint_feature_gt = self.model.batch_joint_feature(X, Y)
 
         try:
             # catch ctrl+c to stop training
@@ -432,21 +439,23 @@ class OneSlackSSVM(BaseSSVM):
                 if self.verbose > 2:
                     print(self)
                 try:
-                    Y_hat, dpsi, loss_mean = self._constraint_from_cache(
-                        X, Y, psi_gt, constraints)
+                    Y_hat, djoint_feature, loss_mean = self._constraint_from_cache(
+                        X, Y, joint_feature_gt, constraints)
                     cached_constraint = True
                 except NoConstraint:
                     try:
-                        Y_hat, dpsi, loss_mean = self._find_new_constraint(
-                            X, Y, psi_gt, constraints)
+                        Y_hat, djoint_feature, loss_mean = self._find_new_constraint(
+                            X, Y, joint_feature_gt, constraints)
                         self._update_cache(X, Y, Y_hat)
                     except NoConstraint:
-                        print("no additional constraints")
+                        if self.verbose:
+                            print("no additional constraints")
                         if (self.switch_to is not None
                                 and self.model.inference_method !=
                                 self.switch_to):
-                            print("Switching to %s inference" %
-                                  str(self.switch_to))
+                            if self.verbose:
+                                print("Switching to %s inference" %
+                                      str(self.switch_to))
                             self.model.inference_method_ = \
                                 self.model.inference_method
                             self.model.inference_method = self.switch_to
@@ -456,12 +465,12 @@ class OneSlackSSVM(BaseSSVM):
 
                 self.timestamps_.append(time() - self.timestamps_[0])
                 self._compute_training_loss(X, Y, iteration)
-                constraints.append((dpsi, loss_mean))
+                constraints.append((djoint_feature, loss_mean))
 
                 # compute primal objective
-                last_slack = -np.dot(self.w, dpsi) + loss_mean
+                last_slack = -np.dot(self.w, djoint_feature) + loss_mean
                 primal_objective = (self.C * len(X)
-                                    * np.max(last_slack, 0)
+                                    * max(last_slack, 0)
                                     + np.sum(self.w ** 2) / 2)
                 self.primal_objective_curve_.append(primal_objective)
                 self.cached_constraint_.append(cached_constraint)
@@ -473,8 +482,8 @@ class OneSlackSSVM(BaseSSVM):
                 if self.cache_tol == "auto" and not cached_constraint:
                     self.cache_tol_ = (primal_objective - objective) / 4
 
-                self.last_slack_ = np.max([(-np.dot(self.w, dpsi) + loss_mean)
-                                           for dpsi, loss_mean in constraints])
+                self.last_slack_ = np.max([(-np.dot(self.w, djoint_feature) + loss_mean)
+                                           for djoint_feature, loss_mean in constraints])
                 self.last_slack_ = max(self.last_slack_, 0)
 
                 if self.verbose > 0:
@@ -493,23 +502,17 @@ class OneSlackSSVM(BaseSSVM):
                     print(self.w)
         except KeyboardInterrupt:
             pass
-        if self.logger is not None:
-            self.logger(self, 'final')
         if self.verbose and self.n_jobs == 1:
             print("calls to inference: %d" % self.model.inference_calls)
         # compute final objective:
-        Y_hat, dpsi, loss_mean = self._find_new_constraint(
-            X, Y, psi_gt, constraints, check=False)
-        last_slack = -np.dot(self.w, dpsi) + loss_mean
-        primal_objective = (self.C * len(X)
-                            * np.max(last_slack, 0)
-                            + np.sum(self.w ** 2) / 2)
-        # we recompute the primal using inference
+        self.timestamps_.append(time() - self.timestamps_[0])
+        primal_objective = self._objective(X, Y)
         self.primal_objective_curve_.append(primal_objective)
-        # add dummy points to objective_curve and timestamps
         self.objective_curve_.append(objective)
         self.cached_constraint_.append(False)
-        self.timestamps_.append(time() - self.timestamps_[0])
+
+        if self.logger is not None:
+            self.logger(self, 'final')
 
         if self.verbose > 0:
             print("final primal objective: %f gap: %f"
