@@ -4,12 +4,13 @@ from tempfile import mkstemp
 from nose.tools import assert_true, assert_equal, assert_less, assert_greater
 
 from sklearn.datasets import load_digits, load_iris
+from sklearn.cross_validation import train_test_split
 
 from pystruct.models import GridCRF, GraphCRF, BinaryClf
 from pystruct.learners import OneSlackSSVM
 from pystruct.datasets import (generate_blocks_multinomial, generate_blocks,
                                generate_checker)
-from pystruct.utils import make_grid_edges, SaveLogger, train_test_split
+from pystruct.utils import make_grid_edges, SaveLogger
 from pystruct.inference import get_installed
 
 # we always try to get the fastest installed inference method
@@ -19,7 +20,6 @@ inference_method = get_installed(["qpbo", "ad3", "lp"])[0]
 def test_multinomial_blocks_one_slack():
     #testing cutting plane ssvm on easy multinomial dataset
     X, Y = generate_blocks_multinomial(n_samples=10, noise=0.5, seed=0)
-    print(np.argmax(X[0], axis=-1))
     n_labels = len(np.unique(Y))
     crf = GridCRF(n_states=n_labels, inference_method=inference_method)
     clf = OneSlackSSVM(model=crf, max_iter=150, C=1,
@@ -81,7 +81,6 @@ def test_constraint_removal():
 def test_binary_blocks_one_slack_graph():
     #testing cutting plane ssvm on easy binary dataset
     # generate graphs explicitly for each example
-    print("testing %s" % inference_method)
     X, Y = generate_blocks(n_samples=3)
     crf = GraphCRF(inference_method=inference_method)
     clf = OneSlackSSVM(model=crf, max_iter=100, C=1,
@@ -117,7 +116,7 @@ def test_one_slack_constraint_caching():
     crf = GridCRF(n_states=n_labels, inference_method='lp')
     clf = OneSlackSSVM(model=crf, max_iter=150, C=1,
                        check_constraints=True, break_on_bad=True,
-                       inference_cache=50, inactive_window=0, verbose=10)
+                       inference_cache=50, inactive_window=0)
     clf.fit(X, Y)
     Y_pred = clf.predict(X)
     assert_array_equal(Y, Y_pred)
@@ -188,4 +187,3 @@ def test_switch_to_ad3():
     # as it might use the relaxation, that is pretty much guraranteed
     assert_greater(ssvm_with_switch.objective_curve_[-1],
                    ssvm.objective_curve_[-1])
-    print(ssvm_with_switch.objective_curve_[-1], ssvm.objective_curve_[-1])
