@@ -1,4 +1,3 @@
-from .backports import train_test_split
 from .inference import (unwrap_pairwise, find_constraint,
                         find_constraint_latent, inference,
                         loss_augmented_inference, objective_primal,
@@ -8,7 +7,7 @@ from .logging import SaveLogger
 from .plotting import plot_grid
 from .graph import make_grid_edges, edge_list_to_features
 
-__all__ = ["train_test_split", "unwrap_pairwise",
+__all__ = ["unwrap_pairwise",
            "make_grid_edges", "find_constraint",
            "find_constraint_latent", "inference", "loss_augmented_inference",
            "objective_primal", "exhaustive_loss_augmented_inference",
